import express from 'express';
import cors from 'cors';
import helmet from 'helmet';
import rateLimit from 'express-rate-limit';
import { config } from '@/utils/config';
import logger from '@/utils/logger';
import { errorHandler } from '@/middleware/errorHandler';
import routes from '@/routes';
import schedulerService from '@/services/schedulerService';

const app = express();

app.use(helmet());

app.use(cors({
  origin: '*',
  credentials: true,
}));

const limiter = rateLimit({
  windowMs: 15 * 60 * 1000, // 15 minutes
  max: 100, // limit each IP to 100 requests per windowMs
  message: {
    success: false,
    message: 'Too many requests from this IP, please try again later.',
  },
});

app.use(limiter);

app.use(express.json({ limit: '10mb' }));
app.use(express.urlencoded({ extended: true, limit: '10mb' }));

app.get('/', (req, res) => {
  res.json({
    success: true,
    message: 'WhatsApp AI Task Scheduling Service',
    data: {
      title: 'WhatsApp AI Task Scheduling Service',
      version: '1.0.0',
      documentation: '/docs',
      health: '/health',
      endpoints: {
        auth: '/api/auth',
        tasks: '/api/tasks',
        health: '/api/health'
      }
    }
  });
});

app.use('/api', routes);

app.get('/docs', (req, res) => {
  res.json({
    success: true,
    message: 'API Documentation',
    data: {
      title: 'WhatsApp AI Task Scheduling Service API',
      version: '1.0.0',
      endpoints: {
        auth: {
          'POST /api/auth/register': 'Register a new user',
          'POST /api/auth/login': 'Login user'
        },
        tasks: {
          'POST /api/tasks': 'Create a new task',
          'GET /api/tasks': 'Get all tasks for user',
          'GET /api/tasks/:id': 'Get specific task',
          'PUT /api/tasks/:id': 'Update task',
          'DELETE /api/tasks/:id': 'Delete task',
          'POST /api/tasks/:id/remind': 'Send WhatsApp reminder'
        },
        health: {
          'GET /api/health': 'Health check endpoint'
        }
      }
    }
  });
});

app.use('*', (req, res) => {
  res.status(404).json({
    success: false,
    message: 'Route not found',
  });
});

app.use(errorHandler);

const gracefulShutdown = () => {
  logger.info('Received shutdown signal, shutting down gracefully...');
  schedulerService.shutdown();
  process.exit(0);
};

process.on('SIGTERM', gracefulShutdown);
process.on('SIGINT', gracefulShutdown);

export default app;