import { Request, Response } from 'express';
import { hashPassword, comparePassword, generateToken } from '@/utils/auth';
import { validate, userRegistrationSchema, userLoginSchema } from '@/utils/validation';
import prisma from '@/utils/database';
import logger from '@/utils/logger';
import { CreateUserData, LoginData, ApiResponse } from '@/types';

export const register = async (req: Request, res: Response) => {
  try {
    const userData: CreateUserData = validate(userRegistrationSchema, req.body);
    
    const existingUser = await prisma.user.findUnique({
      where: { email: userData.email }
    });

    if (existingUser) {
      return res.status(400).json({
        success: false,
        message: 'User with this email already exists'
      } as ApiResponse);
    }

    const hashedPassword = await hashPassword(userData.password);
    
    const user = await prisma.user.create({
      data: {
        email: userData.email,
        password: hashedPassword,
        fullName: userData.fullName,
        phoneNumber: userData.phoneNumber,
        whatsappNumber: userData.whatsappNumber,
      },
      select: {
        id: true,
        email: true,
        fullName: true,
        phoneNumber: true,
        whatsappNumber: true,
        isActive: true,
        createdAt: true,
      }
    });

    const token = generateToken({ id: user.id, email: user.email });

    logger.info(`New user registered: ${user.email}`);

    return res.status(201).json({
      success: true,
      message: 'User registered successfully',
      data: {
        user,
        token
      }
    } as ApiResponse);

  } catch (error) {
    logger.error('Registration error:', error);
    return res.status(400).json({
      success: false,
      message: error instanceof Error ? error.message : 'Registration failed'
    } as ApiResponse);
  }
};

export const login = async (req: Request, res: Response) => {
  try {
    const loginData: LoginData = validate(userLoginSchema, req.body);
    
    const user = await prisma.user.findUnique({
      where: { email: loginData.email }
    });

    if (!user || !user.isActive) {
      return res.status(401).json({
        success: false,
        message: 'Invalid credentials'
      } as ApiResponse);
    }

    const isPasswordValid = await comparePassword(loginData.password, user.password);
    
    if (!isPasswordValid) {
      return res.status(401).json({
        success: false,
        message: 'Invalid credentials'
      } as ApiResponse);
    }

    const token = generateToken({ id: user.id, email: user.email });

    logger.info(`User logged in: ${user.email}`);

    return res.status(200).json({
      success: true,
      message: 'Login successful',
      data: {
        user: {
          id: user.id,
          email: user.email,
          fullName: user.fullName,
          phoneNumber: user.phoneNumber,
          whatsappNumber: user.whatsappNumber,
          isActive: user.isActive,
          createdAt: user.createdAt,
        },
        token
      }
    } as ApiResponse);

  } catch (error) {
    logger.error('Login error:', error);
    return res.status(400).json({
      success: false,
      message: error instanceof Error ? error.message : 'Login failed'
    } as ApiResponse);
  }
};