import OpenAI from 'openai';
import { config } from '@/utils/config';
import logger from '@/utils/logger';

class AIService {
  private client: OpenAI | null = null;

  constructor() {
    if (config.openai.apiKey) {
      this.client = new OpenAI({
        apiKey: config.openai.apiKey,
      });
    } else {
      logger.warn('OpenAI API key not configured. AI functionality disabled.');
    }
  }

  async suggestOptimalTime(dueDate: Date, priority: string): Promise<Date | null> {
    if (!this.client) {
      return this.fallbackScheduling(dueDate, priority);
    }

    try {
      const currentTime = new Date();
      const timeUntilDue = dueDate.getTime() - currentTime.getTime();
      
      if (timeUntilDue <= 0) {
        return new Date(currentTime.getTime() + 60 * 60 * 1000); // 1 hour from now
      }

      const daysDiff = Math.floor(timeUntilDue / (1000 * 60 * 60 * 24));
      const hoursDiff = Math.floor(timeUntilDue / (1000 * 60 * 60));

      const prompt = `
        Based on task management best practices, suggest an optimal time to work on a task with the following details:
        
        Current time: ${currentTime.toISOString()}
        Due date: ${dueDate.toISOString()}
        Priority: ${priority}
        Time available: ${daysDiff} days, ${hoursDiff % 24} hours
        
        Consider:
        - Task priority (urgent tasks should be scheduled sooner)
        - Optimal work hours (9 AM - 5 PM on weekdays)
        - Buffer time before due date
        - Work-life balance
        
        Respond with only the suggested datetime in ISO format: YYYY-MM-DDTHH:MM:SS.000Z
      `;

      const response = await this.client.chat.completions.create({
        model: 'gpt-3.5-turbo',
        messages: [
          { role: 'system', content: 'You are a task scheduling AI assistant.' },
          { role: 'user', content: prompt }
        ],
        max_tokens: 50,
        temperature: 0.3,
      });

      const suggestedTimeStr = response.choices[0]?.message?.content?.trim();
      if (!suggestedTimeStr) {
        return this.fallbackScheduling(dueDate, priority);
      }

      const suggestedTime = new Date(suggestedTimeStr);
      
      if (isNaN(suggestedTime.getTime()) || suggestedTime < currentTime) {
        return this.fallbackScheduling(dueDate, priority);
      }
      
      if (suggestedTime > dueDate) {
        return new Date(dueDate.getTime() - 2 * 60 * 60 * 1000); // 2 hours before due
      }

      return suggestedTime;

    } catch (error) {
      logger.error('AI scheduling failed:', error);
      return this.fallbackScheduling(dueDate, priority);
    }
  }

  private fallbackScheduling(dueDate: Date, priority: string): Date {
    const currentTime = new Date();
    const timeUntilDue = dueDate.getTime() - currentTime.getTime();

    let hoursToAdd: number;
    
    switch (priority) {
      case 'URGENT':
        hoursToAdd = 1;
        break;
      case 'HIGH':
        hoursToAdd = Math.min(6, timeUntilDue / (1000 * 60 * 60) / 4);
        break;
      case 'MEDIUM':
        hoursToAdd = Math.min(24, timeUntilDue / (1000 * 60 * 60) / 2);
        break;
      case 'LOW':
      default:
        hoursToAdd = Math.min(48, (timeUntilDue / (1000 * 60 * 60)) * 0.75);
        break;
    }

    return new Date(currentTime.getTime() + hoursToAdd * 60 * 60 * 1000);
  }

  async generateTaskInsights(tasks: any[]): Promise<string> {
    if (!this.client) {
      return 'AI insights not available. Please configure OpenAI API key.';
    }

    try {
      const taskSummary = tasks.map(task => 
        `- ${task.title} (Priority: ${task.priority}, Status: ${task.status})`
      ).join('\n');

      const prompt = `
        Analyze the following task list and provide productivity insights:
        
        ${taskSummary}
        
        Provide insights on:
        1. Task distribution by priority
        2. Potential bottlenecks
        3. Productivity recommendations
        4. Time management suggestions
        
        Keep the response concise and actionable (max 200 words).
      `;

      const response = await this.client.chat.completions.create({
        model: 'gpt-3.5-turbo',
        messages: [
          { role: 'system', content: 'You are a productivity analysis AI assistant.' },
          { role: 'user', content: prompt }
        ],
        max_tokens: 250,
        temperature: 0.7,
      });

      return response.choices[0]?.message?.content?.trim() || 'Unable to generate insights at this time.';

    } catch (error) {
      logger.error('AI insights generation failed:', error);
      return 'Unable to generate insights at this time.';
    }
  }
}

const aiService = new AIService();

export const suggestOptimalTime = async (dueDate: Date, priority: string): Promise<Date | null> => {
  return aiService.suggestOptimalTime(dueDate, priority);
};

export const generateTaskInsights = async (tasks: any[]): Promise<string> => {
  return aiService.generateTaskInsights(tasks);
};

export default aiService;