import cron from 'node-cron';
import prisma from '@/utils/database';
import logger from '@/utils/logger';
import { sendTaskReminder, sendTaskOverdue } from './whatsappService';

class SchedulerService {
  private jobs: Map<string, cron.ScheduledTask> = new Map();

  constructor() {
    this.setupRecurringJobs();
  }

  private setupRecurringJobs() {
    const dailyReminderJob = cron.schedule('0 9 * * *', async () => {
      await this.sendDailyReminders();
    }, {
      scheduled: false,
      timezone: 'UTC'
    });

    const overdueCheckJob = cron.schedule('0 18 * * *', async () => {
      await this.checkOverdueTasks();
    }, {
      scheduled: false,
      timezone: 'UTC'
    });

    this.jobs.set('daily_reminders', dailyReminderJob);
    this.jobs.set('overdue_check', overdueCheckJob);

    dailyReminderJob.start();
    overdueCheckJob.start();

    logger.info('Scheduler service initialized with recurring jobs');
  }

  private async sendDailyReminders() {
    logger.info('Starting daily reminder job');
    
    try {
      const today = new Date();
      const tomorrow = new Date(today);
      tomorrow.setDate(today.getDate() + 1);
      
      today.setHours(0, 0, 0, 0);
      tomorrow.setHours(0, 0, 0, 0);

      const tasks = await prisma.task.findMany({
        where: {
          status: {
            in: ['PENDING', 'IN_PROGRESS']
          },
          scheduledAt: {
            gte: today,
            lt: tomorrow
          },
          whatsappReminderSent: false,
          owner: {
            whatsappNumber: {
              not: null
            }
          }
        },
        include: {
          owner: true
        }
      });

      let sentCount = 0;
      for (const task of tasks) {
        if (task.owner.whatsappNumber && await sendTaskReminder(task.owner.whatsappNumber, task)) {
          await prisma.task.update({
            where: { id: task.id },
            data: { whatsappReminderSent: true }
          });
          sentCount++;
        }
      }

      logger.info(`Sent ${sentCount} daily reminders`);
    } catch (error) {
      logger.error('Error in daily reminder job:', error);
    }
  }

  private async checkOverdueTasks() {
    logger.info('Starting overdue task check');
    
    try {
      const now = new Date();
      
      const overdueTasks = await prisma.task.findMany({
        where: {
          status: {
            in: ['PENDING', 'IN_PROGRESS']
          },
          dueDate: {
            lt: now
          },
          owner: {
            whatsappNumber: {
              not: null
            }
          }
        },
        include: {
          owner: true
        }
      });

      let sentCount = 0;
      for (const task of overdueTasks) {
        if (task.owner.whatsappNumber && await sendTaskOverdue(task.owner.whatsappNumber, task)) {
          sentCount++;
        }
      }

      logger.info(`Sent ${sentCount} overdue notifications`);
    } catch (error) {
      logger.error('Error in overdue check job:', error);
    }
  }

  scheduleTaskReminder(taskId: number, remindAt: Date) {
    const jobId = `task_reminder_${taskId}`;
    
    if (this.jobs.has(jobId)) {
      this.jobs.get(jobId)?.stop();
      this.jobs.delete(jobId);
    }

    const job = cron.schedule(this.dateToCron(remindAt), async () => {
      await this.sendIndividualReminder(taskId);
      this.jobs.delete(jobId);
    }, {
      scheduled: false,
      timezone: 'UTC'
    });

    this.jobs.set(jobId, job);
    job.start();

    logger.info(`Scheduled reminder for task ${taskId} at ${remindAt.toISOString()}`);
  }

  private async sendIndividualReminder(taskId: number) {
    try {
      const task = await prisma.task.findUnique({
        where: { id: taskId },
        include: { owner: true }
      });

      if (task && 
          task.status !== 'COMPLETED' && 
          task.status !== 'CANCELLED' && 
          task.owner.whatsappNumber) {
        
        const success = await sendTaskReminder(task.owner.whatsappNumber, task);
        if (success) {
          await prisma.task.update({
            where: { id: taskId },
            data: { whatsappReminderSent: true }
          });
        }
      }
    } catch (error) {
      logger.error(`Error sending individual reminder for task ${taskId}:`, error);
    }
  }

  private dateToCron(date: Date): string {
    const minute = date.getUTCMinutes();
    const hour = date.getUTCHours();
    const day = date.getUTCDate();
    const month = date.getUTCMonth() + 1;
    
    return `${minute} ${hour} ${day} ${month} *`;
  }

  shutdown() {
    for (const [jobId, job] of this.jobs) {
      job.stop();
      logger.info(`Stopped job: ${jobId}`);
    }
    this.jobs.clear();
    logger.info('Scheduler service shut down');
  }
}

export default new SchedulerService();